#downsample first    
#convert tiff files and extract a boundary box around the area you want
import pandas as pd
import numpy as np
import os
from scipy.ndimage import convolve
import glob
from PIL import Image

#run this from within the 'downsampled' folder

lat_range = np.arange(0, 75,  0.008333333333333333)
lon_range = np.arange(-180, -60,  0.008333333333333333)

Image.MAX_IMAGE_PIXELS = 600000000

years = ['2014','2015','2016','2017','2018','2019','2020']

for year in years:
    os.chdir('/p/projects/compacts/data/nightlight/viirs_monthly/'+year)
    big_df = pd.DataFrame()
    for file in glob.glob('/p/projects/compacts/data/nightlight/'+year+'/*.tif'):
        image = Image.open(file)
        array = np.array(image)
        array_downsampled = convolve(array, np.array([[0.25,0.25],[0.25,0.25]]))[:array.shape[0]:2,:array.shape[1]:2]
        pre_df = pd.DataFrame(array_downsampled)
        pre_df.columns = lon_range
        keeplist = [i for i in pre_df.columns if (i<=-93) &(i>=-107)]   #smaller longitude range
        droplist = [i for i in pre_df.columns if i not in keeplist]
        pre_df.drop(droplist, axis =1, inplace=True)
        pre_df.index = lat_range
        pre_df = pre_df[(25<=pre_df.index) & (pre_df.index <= 37)] #smaller latitude range
        df = pre_df.unstack()
        df = df.reset_index()
        df.rename({'level_0':'lon_s', 'level_1':'lat_s', 0:'light'}, axis =1, inplace=True)
        print ('year and area downsampled')
        big_df[str(file.split('_')[-6].split('-')[0][-4:-2])] = df.light
        print ('file '+str(file.split('_')[-6].split('-')[0][-4:-2]) +' processed')
    big_df['lon_s'] = df['lon_s']
    big_df['lat_s'] = df['lat_s']
    big_df['lon_e'] = big_df['lon_s']+ 0.008333333333333333
    big_df['lat_e'] = big_df['lat_s']+ 0.008333333333333333
    big_df['year'] = year
    big_df.rename({'01':'Jan','02':'Feb','03':'Mar','04':'Apr','05':'May','06':'Jun','07':'Jul','08':'Aug','09':'Sep','10':'Oct','11':'Nov','12':'Dec'}, axis =1, inplace=True)
    big_df.to_csv(year+'_monthly_txbox_downsample.csv', encoding = 'utf-8', index=False)
    print ('year saved!')
    os.chdir('../')

#transform these all to geo and bound by area
import geopandas as gpd
from shapely.geometry import Point, Polygon, LineString

counties = gpd.read_file('/p/projects/impactee/kelsey/night_light_data/viirs_monthly/monthly/fl.shp',crs= {'init': 'epsg:4326'})
counties['FIPS'] = counties['FIPS'].astype(int)
cs = counties[['geometry', 'FIPS']].to_crs(epsg=4326)
print (cs.crs)
print (cs.geometry)

years = ['2014','2015','2016','2017','2018','2019','2020']
for year in years:
    for file in glob.glob('/p/projects/compacts/data/nightlight/viirs_monthly/'+year+'/'+year+'_monthly_irmabox_cf.csv'):
        df = pd.read_csv(file)
        geometry = df.apply(lambda row: Polygon([(row['lon_s'], row['lat_s']), (row['lon_s'], row['lat_e']), (row['lon_e'], row['lat_e']), (row['lon_e'], row['lat_s'])]), axis=1)
        geodf = gpd.GeoDataFrame(df, crs= {'init': 'epsg:4326'}, geometry= geometry).to_crs(epsg=4326)
        geodf = geodf.set_crs('epsg:4326')
        print (geodf.crs)
        print (geodf.geometry)
        bind = gpd.sjoin(geodf, cs, how= 'inner')
        print ('files bound by geography')
        print (bind.crs)
        print (bind.geometry)
        bind.to_file('geofiles/'+year+'_monthly_hermine_cf_geo.shp')
        print ('files saved')

